@@ -293,6 +293,30 @@ class VariableClUpdateMetadataForce(Variable):
                 ("skip", _("Skip")),
                 ("auto", _("If needed"))]
 
+
+class VariableClUpdateEgencacheForce(Variable):
+    """
+    Принудительное выполнение egencache
+    """
+    type = "choice"
+    value = "auto"
+    opt = ["--egencache"]
+    syntax = "--{choice}-egencache"
+    metavalue = "MODE"
+    #untrusted = True
+
+    def init(self):
+        self.help = ("'force' - " + _("force the update overlays cache") +
+                     ",\n'skip' - " + _("skip the overlays cache update") +
+                     ",\n'auto' - " + _("update overlays cache if it is outdated"))
+        self.label = _("Update overlays cache")
+
+    def choice(self):
+        return [("force", _("Force")),
+                ("skip", _("Skip")),
+                ("auto", _("If needed"))]
+
+
 class VariableClUpdateEixupdateForce(Variable):
     """
     Принудительное действие с eix-update
@@ -800,7 +824,7 @@ class VariableClUpdateProfileSystem(Variable):
                 if not dv.Get('os_linux_name'):
                     raise VariableError()
             except (DataVarsError, VariableError) as e:
-                raise VariableError(_("Profile is not Calculate"))
+                raise VariableError(_("The selected profile is not Calculate"))
         else:
             raise VariableError(_("Wrong Calculate profile"))
 
@@ -871,8 +895,8 @@ class VariableClUpdateProfileSyncSet(Variable):
     opt = ["-s", "--sync"]
 
     def init(self):
-        self.label = _("Synchronize repositories")
-        self.help = _("synchronize repositories")
+        self.label = _("Repositories to be synchronized")
+        self.help = _("repositories to be synchronized")
 
 
 class VariableClUpdateAutocheckSet(Variable):
@@ -916,8 +940,8 @@ class VariableClUpdateAutocheckScheduleSet(Variable):
     opt = ["--schedule"]
 
     def init(self):
-        self.label = _("Consider auto-check schedule")
-        self.help = "consider auto-check schedule"
+        self.label = _("Consider the autocheck schedule")
+        self.help = _("consider the autocheck schedule")
 
 class VariableClUpdateKernelVersion(ReadonlyVariable):
     """
